function [f, icc] = aprocsa(scores, patient)

% aprocsa  Analyze APROCSA data
%
%   F = APROCSA(SCORES) returns the four factor scores for the features in
%   SCORES, which is an NP * NF matrix where NP is the number of patients
%   and NF is the number of scores. The number of scores can be 27 (all
%   features) or 23 (only the features included in the factor analysis).
%
%   [F, ICC] = APROCSA(SCORES, PATIENT) compares the SCORES for the named
%   PATIENT to the mean of the experienced researcher ratings, and returns
%   the four factor scores as well as the ICC (type A,1).
%
%   Examples:
%
%     1. To obtain factor scores for a patient with the following APROCSA
%        feature scores:
%
%        aprocsa([3 1 1 0 1 0 0 0 0 3 3 3 1 1 3 1 2 1 0 0 0 3 0 0 0 3 3]);
%
%     2. To determine how well your scores for patient TCU08a match the
%        experienced researcher scores for this patient:
%
%        aprocsa([3 1 1 0 1 0 0 0 0 3 3 3 1 1 3 1 2 1 0 0 0 3 0 0 0 3 3], 'tcu08a');
%
%   Version 1.0; Mar 21, 2018
%   Copyright 2018 Stephen M. Wilson


% check for correct number of input arguments
if nargin < 1, error('No scores provided.'); end
if nargin < 2, refMode = false; else refMode = true; end % refMode = compare to reference data

data_fname = 'aprocsa_data.mat';
if ~exist(data_fname, 'file')
  data_fname = fullfile(fileparts(which(mfilename)), data_fname);
  if ~exist(data_fname, 'file')
    error('Cannot find aprocsa_data.mat; it needs to be located in the current directory or the same directory as aprocsa.m.');
  end
end
data = load(data_fname);

% check size of scores
if ndims(scores) > 2 %#ok<ISMAT>
  error('SCORES should be a vector of scores for one patient or a matrix of scores for multiple patients.');
end
if all(size(scores) == [27 1]) % flip vector for this simple case
  scores = scores';
end
if all(size(scores) == [23 1]) % flip vector for this simple case
  scores = scores';
end
[np, nf] = size(scores);
if nf ~= 23  && nf ~= 27
  error('SCORES should be a NP * NF matrix where NP is the number of patients and NF is the number of features (23 or 27).');
end

% calculate factor scores
if nf == 27
  scores23 = scores(:, data.fa_include);
else
  scores23 = scores;
end
f = (scores23 - repmat(mean(data.fa_matrix), np, 1)) ./ ...
  repmat(std(data.fa_matrix), np, 1) * ...
  diag(1 ./ sqrt(data.psi)) / (data.lambda' * diag(1 ./ sqrt(data.psi)));

% ICC if patient specified
if refMode
  if np > 1
    error('Only one set of scores can be provided when comparing to a named patient.');
  end
  % find the patient
  pat = 0;
  for p = 1:numel(data.patients)
    if strcmpi(patient, data.patients{p})
      pat = p;
      continue;
    end
  end
  if ~pat
    error('Patient %s does not match any of the known patients.', patient);
  end
  % get patient scores
  expert_scores = data.researcher_ratings(pat, :);
  % reduce to 23 features if only 23 provided
  if nf == 23
    expert_scores = expert_scores(:, data.fa_include);
  end
  % calculate ICC for each row of input scores (normally there would be just one)
  icc = ICC([expert_scores', scores'], 'A-1');

  % output
  fprintf('+-----------------------------------------------------------+\n');
  fprintf('| Patient: %-48s |\n', data.patients{pat});
  fprintf('+-----------------------------------+-------+-------+-------+\n');
  fprintf('| %-33s | %s | %s | %s |\n', 'Feature/factor', 'Input', 'Casil', ' Diff');
  fprintf('+-----------------------------------+-------+-------+-------+\n');
  if nf == 27
    for v = 1:27
      fprintf('| %-33s | %5.2f | %5.2f | %5.2f |\n', data.features{v}, ...
        scores(1, v), expert_scores(v), scores(1, v) - expert_scores(v));
    end
  else
    for v = 1:23
      fprintf('| %-33s | %5.2f | %5.2f | %5.2f |\n', data.fa_features{v}, ...
        scores(1, v), expert_scores(v), scores(1, v) - expert_scores(v));
    end
  end
  fprintf('+-----------------------------------+-------+-------+-------+\n');
  for v = 1:4
    fprintf('| %-33s | %5.2f | %5.2f | %5.2f |\n', ...
      data.factors{v}, f(1, v), data.f(pat, v), f(1, v) - data.f(pat, v));
  end
  fprintf('+-----------------------------------+-------+-------+-------+\n');
  fprintf('| %-33s |       r = %5.2f       |\n', 'ICC', icc);
  fprintf('+-----------------------------------+-----------------------+\n');

else
  % output
  for p = 1:np
    fprintf('+-------------------------------------------+\n');
    fprintf('| Patient: %-10d                       |\n', p);
    fprintf('+-----------------------------------+-------+\n');
    fprintf('| %-33s | %s |\n', 'Feature/factor', 'Input');
    fprintf('+-----------------------------------+-------+\n');
    if nf == 27
      for v = 1:27
        fprintf('| %-33s | %5.2f |\n', data.features{v}, scores(p, v));
      end
    else
      for v = 1:23
        fprintf('| %-33s | %5.2f |\n', data.fa_features{v}, scores(p, v));
      end
    end
    fprintf('+-----------------------------------+-------+\n');
    for v = 1:4
      fprintf('| %-33s | %5.2f |\n', data.factors{v}, f(p, v));
    end
    fprintf('+-----------------------------------+-------+\n');
  end
end


function [r, LB, UB, F, df1, df2, p] = ICC(M, type, alpha, r0)
% Intraclass correlation
%   [r, LB, UB, F, df1, df2, p] = ICC(M, type, alpha, r0)
%
%   M is matrix of observations. Each row is an object of measurement and
%   each column is a judge or measurement.
%
%   'type' is a string that can be one of the six possible codes for the desired
%   type of ICC:
%       '1-1': The degree of absolute agreement among measurements made on
%         randomly seleted objects. It estimates the correlation of any two
%         measurements.
%       '1-k': The degree of absolute agreement of measurements that are
%         averages of k independent measurements on randomly selected
%         objects.
%       'C-1': case 2: The degree of consistency among measurements. Also known
%         as norm-referenced reliability and as Winer's adjustment for
%         anchor points. case 3: The degree of consistency among measurements maded under
%         the fixed levels of the column factor. This ICC estimates the
%         corrlation of any two measurements, but when interaction is
%         present, it underestimates reliability.
%       'C-k': case 2: The degree of consistency for measurements that are
%         averages of k independent measurements on randomly selected
%         onbjectgs. Known as Cronbach's alpha in psychometrics. case 3:  
%         The degree of consistency for averages of k independent
%         measures made under the fixed levels of column factor.
%       'A-1': case 2: The degree of absolute agreement among measurements. Also
%         known as criterion-referenced reliability. case 3: The absolute 
%         agreement of measurements made under the fixed levels of the column factor.
%       'A-k': case 2: The degree of absolute agreement for measurements that are
%         averages of k independent measurements on randomly selected objects.
%         case 3: he degree of absolute agreement for measurements that are
%         based on k independent measurements maded under the fixed levels
%         of the column factor.
%
%       ICC is the estimated intraclass correlation. LB and UB are upper
%       and lower bounds of the ICC with alpha level of significance. 
%
%       In addition to estimation of ICC, a hypothesis test is performed
%       with the null hypothesis that ICC = r0. The F value, degrees of
%       freedom and the corresponding p-value of the this test are
%       reported.
%
%       Reference: McGraw, K. O., Wong, S. P., "Forming Inferences About
%       Some Intraclass Correlation Coefficients", Psychological Methods,
%       Vol. 1, No. 1, pp. 30-46, 1996
%
% Copyright (c) 2016, Arash Salarian
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

if nargin < 3
    alpha = .05;
end

if nargin < 4
    r0 = 0;
end

[n, k] = size(M);

SStotal = var(M(:)) *(n*k - 1);
MSR = var(mean(M, 2)) * k;
MSW = sum(var(M,0, 2)) / n;
MSC = var(mean(M, 1)) * n;
MSE = (SStotal - MSR *(n - 1) - MSC * (k -1))/ ((n - 1) * (k - 1));

switch type
    case '1-1'
        [r, LB, UB, F, df1, df2, p] = ICC_case_1_1(MSR, MSE, MSC, MSW, alpha, r0, n, k);
    case '1-k'
        [r, LB, UB, F, df1, df2, p] = ICC_case_1_k(MSR, MSE, MSC, MSW, alpha, r0, n, k);
    case 'C-1'
        [r, LB, UB, F, df1, df2, p] = ICC_case_C_1(MSR, MSE, MSC, MSW, alpha, r0, n, k);
    case 'C-k'
        [r, LB, UB, F, df1, df2, p] = ICC_case_C_k(MSR, MSE, MSC, MSW, alpha, r0, n, k);
    case 'A-1'
        [r, LB, UB, F, df1, df2, p] = ICC_case_A_1(MSR, MSE, MSC, MSW, alpha, r0, n, k);
    case 'A-k'
        [r, LB, UB, F, df1, df2, p] = ICC_case_A_k(MSR, MSE, MSC, MSW, alpha, r0, n, k);
end


%----------------------------------------
function [r, LB, UB, F, df1, df2, p] = ICC_case_1_1(MSR, MSE, MSC, MSW, alpha, r0, n, k)
r = (MSR - MSW) / (MSR + (k-1)*MSW);

F = (MSR/MSW) * (1-r0)/(1+(k-1)*r0);
df1 = n-1;
df2 = n*(k-1);
p = 1-fcdf(F, df1, df2);

FL = (MSR/MSW) / finv(1-alpha/2, n-1, n*(k-1));
FU = (MSR/MSW) * finv(1-alpha/2, n*(k-1), n-1);

LB = (FL - 1) / (FL + (k-1));
UB = (FU - 1) / (FU + (k-1));

%----------------------------------------
function [r, LB, UB, F, df1, df2, p] = ICC_case_1_k(MSR, MSE, MSC, MSW, alpha, r0, n, k)
r = (MSR - MSW) / MSR;

F = (MSR/MSW) * (1-r0);
df1 = n-1;
df2 = n*(k-1);
p = 1-fcdf(F, df1, df2);

FL = (MSR/MSW) / finv(1-alpha/2, n-1, n*(k-1));
FU = (MSR/MSW) * finv(1-alpha/2, n*(k-1), n-1);

LB = 1 - 1 / FL;
UB = 1 - 1 / FU;

%----------------------------------------
function [r, LB, UB, F, df1, df2, p] = ICC_case_C_1(MSR, MSE, MSC, MSW, alpha, r0, n, k)
r = (MSR - MSE) / (MSR + (k-1)*MSE);

F = (MSR/MSE) * (1-r0)/(1+(k-1)*r0);
df1 = n - 1;
df2 = (n-1)*(k-1);
p = 1-fcdf(F, df1, df2);

FL = (MSR/MSE) / finv(1-alpha/2, n-1, (n-1)*(k-1));
FU = (MSR/MSE) * finv(1-alpha/2, (n-1)*(k-1), n-1);

LB = (FL - 1) / (FL + (k-1));
UB = (FU - 1) / (FU + (k-1));

%----------------------------------------
function [r, LB, UB, F, df1, df2, p] = ICC_case_C_k(MSR, MSE, MSC, MSW, alpha, r0, n, k)
r = (MSR - MSE) / MSR;

F = (MSR/MSE) * (1-r0);
df1 = n - 1;
df2 = (n-1)*(k-1);
p = 1-fcdf(F, df1, df2);

FL = (MSR/MSE) / finv(1-alpha/2, n-1, (n-1)*(k-1));
FU = (MSR/MSE) * finv(1-alpha/2, (n-1)*(k-1), n-1);

LB = 1 - 1 / FL;
UB = 1 - 1 / FU;

%----------------------------------------
function [r, LB, UB, F, df1, df2, p] = ICC_case_A_1(MSR, MSE, MSC, MSW, alpha, r0, n, k)
r = (MSR - MSE) / (MSR + (k-1)*MSE + k*(MSC-MSE)/n);

a = (k*r0) / (n*(1-r0));
b = 1 + (k*r0*(n-1))/(n*(1-r0));
F = MSR / (a*MSC + b*MSE);
%df2 = (a*MSC + b*MSE)^2/((a*MSC)^2/(k-1) + (b*MSE)^2/((n-1)*(k-1)));

a = k*r/(n*(1-r));
b = 1+k*r*(n-1)/(n*(1-r));
v = (a*MSC + b*MSE)^2/((a*MSC)^2/(k-1) + (b*MSE)^2/((n-1)*(k-1)));

df1 = n - 1;
df2 = v;
p = 1-fcdf(F, df1, df2);

Fs = finv(1-alpha/2, n-1, v);
LB = n*(MSR - Fs*MSE)/(Fs*(k*MSC + (k*n - k - n)*MSE) + n*MSR);

Fs = finv(1-alpha/2, v, n-1);
UB = n*(Fs*MSR-MSE)/(k*MSC + (k*n - k - n)*MSE + n*Fs*MSR);

%----------------------------------------
function [r, LB, UB, F, df1, df2, p] = ICC_case_A_k(MSR, MSE, MSC, MSW, alpha, r0, n, k)
r = (MSR - MSE) / (MSR + (MSC-MSE)/n);

c = r0/(n*(1-r0));
d = 1 + (r0*(n-1))/(n*(1-r0));
F = MSR / (c*MSC + d*MSE);
%df2 = (c*MSC + d*MSE)^2/((c*MSC)^2/(k-1) + (d*MSE)^2/((n-1)*(k-1)));

a = k*r/(n*(1-r));
b = 1+k*r*(n-1)/(n*(1-r));
v = (a*MSC + b*MSE)^2/((a*MSC)^2/(k-1) + (b*MSE)^2/((n-1)*(k-1)));

df1 = n - 1;
df2 = v;
p = 1-fcdf(F, df1, df2);

Fs = finv(1-alpha/2, n-1, v);
LB = n*(MSR - Fs*MSE)/(Fs*(MSC-MSE) + n*MSR);

Fs = finv(1-alpha/2, v, n-1);
UB = n*(Fs*MSR - MSE)/(MSC - MSE + n*Fs*MSR);
